@@ -1,23 +1,27 @@
 import collections
 import threading
-from typing import Optional
+from typing import Optional, Union
 
 from .retryable_logs import RetryableLogs
 from .evaluation_details import EvaluationDetails
 from .evaluator import _ConfigEvaluation
 from .statsig_event import StatsigEvent
+# from .statsig_user import StatsigUser
 from .layer import Layer
 from .utils import logger
 from .thread_util import spawn_background_thread, THREAD_JOIN_TIMEOUT
+from .diagnostics import _Diagnostics
 
 _CONFIG_EXPOSURE_EVENT = "statsig::config_exposure"
 _LAYER_EXPOSURE_EVENT = "statsig::layer_exposure"
 _GATE_EXPOSURE_EVENT = "statsig::gate_exposure"
+_DIAGNOSTICS_EVENT = "statsig::diagnostics"
 
+_IGNORED_METADATA_KEYS = {'serverTime', 'configSyncTime', 'initTime', 'reason'}
 
 def _safe_add_evaluation_to_event(
-        evaluation_details: EvaluationDetails, event: StatsigEvent):
-    if evaluation_details is None:
+        evaluation_details: Union[EvaluationDetails, None], event: StatsigEvent):
+    if evaluation_details is None or event is None or event.metadata is None:
         return
 
     event.metadata["reason"] = evaluation_details.reason
@@ -29,10 +33,12 @@ def _safe_add_evaluation_to_event(
 class _StatsigLogger:
     _background_flush: Optional[threading.Thread]
     _background_retry: Optional[threading.Thread]
+    _background_deduper: Optional[threading.Thread]
 
     def __init__(self, net, shutdown_event, statsig_metadata, error_boundary, options):
         self._events = []
         self._retry_logs = collections.deque(maxlen=10)
+        self._deduper = set()
         self._net = net
         self._statsig_metadata = statsig_metadata
         self._local_mode = options.local_mode
@@ -44,6 +50,7 @@ class _StatsigLogger:
         self._shutdown_event = shutdown_event
         self._background_flush = None
         self._background_retry = None
+        self._background_deduper = None
         self.spawn_bg_threads_if_needed()
 
     def spawn_bg_threads_if_needed(self):
@@ -58,12 +65,16 @@ class _StatsigLogger:
             self._background_retry = spawn_background_thread(
                 self._periodic_retry, (self._shutdown_event,), self._error_boundary)
 
+        if self._background_deduper is None or not self._background_deduper.is_alive():
+            self._background_deduper = spawn_background_thread(
+                self._periodic_dedupe_clear, (self._shutdown_event,), self._error_boundary)
+
     def log(self, event):
         if self._local_mode:
             return
         self._events.append(event.to_dict())
         if len(self._events) >= self._event_queue_size:
-            self._flush()
+            self.flush()
 
     def log_gate_exposure(self, user, gate, value, rule_id, secondary_exposures,
                           evaluation_details: EvaluationDetails, is_manual_exposure=False):
@@ -73,6 +84,9 @@ class _StatsigLogger:
             "gateValue": "true" if value else "false",
             "ruleID": rule_id,
         }
+        if not self._is_unique_exposure(user, _GATE_EXPOSURE_EVENT, event.metadata):
+            return
+
         if is_manual_exposure:
             event.metadata["isManualExposure"] = "true"
 
@@ -90,6 +104,8 @@ class _StatsigLogger:
             "config": config,
             "ruleID": rule_id,
         }
+        if not self._is_unique_exposure(user, _CONFIG_EXPOSURE_EVENT, event.metadata):
+            return
         if is_manual_exposure:
             event.metadata["isManualExposure"] = "true"
 
@@ -111,13 +127,16 @@ class _StatsigLogger:
             exposures = config_evaluation.secondary_exposures
             allocated_experiment = config_evaluation.allocated_experiment
 
-        event.metadata = {
+        metadata = {
             "config": layer.name,
             "ruleID": layer.rule_id,
             "allocatedExperiment": allocated_experiment,
             "parameterName": parameter_name,
             "isExplicitParameter": "true" if is_explicit else "false"
         }
+        if not self._is_unique_exposure(user, _LAYER_EXPOSURE_EVENT, metadata):
+            return
+        event.metadata = metadata
         if is_manual_exposure:
             event.metadata["isManualExposure"] = "true"
 
@@ -125,9 +144,10 @@ class _StatsigLogger:
 
         _safe_add_evaluation_to_event(
             config_evaluation.evaluation_details, event)
+
         self.log(event)
 
-    def _flush(self):
+    def flush(self):
         if len(self._events) == 0:
             return
         events_copy = self._events.copy()
@@ -140,7 +160,7 @@ class _StatsigLogger:
             self._retry_logs.append(RetryableLogs(res, 0))
 
     def shutdown(self):
-        self._flush()
+        self.flush()
 
         if self._background_flush is not None:
             self._background_flush.join(THREAD_JOIN_TIMEOUT)
@@ -153,7 +173,16 @@ class _StatsigLogger:
             try:
                 if shutdown_event.wait(self._logging_interval):
                     break
-                self._flush()
+                self.flush()
+            except Exception as e:
+                self._error_boundary.log_exception(e)
+
+    def _periodic_dedupe_clear(self, shutdown_event):
+        while True:
+            try:
+                if shutdown_event.wait(self._logging_interval):
+                    break
+                self._deduper = set()
             except Exception as e:
                 self._error_boundary.log_exception(e)
 
@@ -161,7 +190,6 @@ class _StatsigLogger:
         while True:
             if shutdown_event.wait(self._retry_interval):
                 break
-
             length = len(self._retry_logs)
             for _i in range(length):
                 try:
@@ -178,3 +206,30 @@ class _StatsigLogger:
                         return
 
                     self._retry_logs.append(RetryableLogs(retry_logs.payload, retry_logs.retries))
+
+
+    def log_diagnostics_event(self, diagnostics: _Diagnostics):
+        event = StatsigEvent(None, _DIAGNOSTICS_EVENT)
+        event.metadata = diagnostics.serialize()
+        self.log(event)
+
+    def _is_unique_exposure(self, user, eventName: str, metadata: dict or None) -> bool:
+        if user is None:
+            return True
+        if len(self._deduper) > 10000:
+            self._deduper = set()
+        custom_id_key = ''
+        if user.custom_ids and isinstance(user.custom_ids, dict):
+            custom_id_key = ','.join(user.custom_ids.values())
+
+        metadata_key = ''
+        if metadata and isinstance(metadata, dict):
+            metadata_key = ','.join(str(value) for key, value in metadata.items() if key not in _IGNORED_METADATA_KEYS)
+
+        key = ','.join(str(item) for item in [user.user_id, custom_id_key, eventName, metadata_key])
+
+        if key in self._deduper:
+            return False
+
+        self._deduper.add(key)
+        return True
